#define FLITE_PROJECT_NAME "Festival Lite"
#define FLITE_PROJECT_PREFIX "flite"
#define FLITE_PROJECT_VERSION "1.2"
#define FLITE_PROJECT_DATE "Feb 2003"
#define FLITE_PROJECT_STATE "release"
#define FLITE_PROJECT_SHLIB_VERSION "1"
